
import re

import subprocess
import tempfile

version_regex = re.compile('bibutils suite version (.*) date', re.MULTILINE)

def checkBibutilsVersion(min_version=4.6):
    """ Ensure that a certain Bibutils version is installed"""

    fp_out = tempfile.mktemp()
    pipe = subprocess.Popen('bib2xml --version', shell=True, stderr=subprocess.PIPE)
    output = pipe.stderr.read()
    mo = version_regex.search(output)
    if mo:
        s = mo.group(1)

        try:
            version = float(s)
        except TypeError:
            version = None
        if version and version >= min_version: 
            return version
        else:
            raise RuntimeError('Minimum requirement is Bibtutils %s, found %s' % (min_version, version))

    raise RuntimeError('Unable to determine Bibtutils version')




import unittest, doctest

from zope.testing import doctestunit
from zope.component import testing
from zope.component import provideUtility
from zope.component import getUtility

from zope.app.schema.vocabulary import IVocabularyFactory

from bibliograph.core.vocabulary import BibFormatVocabularyFactory

class DummyEntry(object):

    def __init__(self, id=None):
        self.id = id

    def getId(self):
        return self.id

class VocabularyTestCase(unittest.TestCase):

    def setUp(self):
        provideUtility(BibFormatVocabularyFactory,
                       IVocabularyFactory,
                       u'bibliography.formats')

    def test_formatsvoc(self):
        voc = getUtility(IVocabularyFactory, u'bibliography.formats')(object())
        assert 'bibtex' in voc


def test_suite():
    return unittest.TestSuite([

        # Unit tests for your API
        doctestunit.DocFileSuite(
            'README.txt',
            package='bibliograph.core',
            setUp=testing.setUp,
            tearDown=testing.tearDown),

        doctestunit.DocTestSuite(
            'bibliograph.core.utils',
            optionflags=doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE,
            globs=dict(DummyEntry=DummyEntry)),

        unittest.makeSuite(VocabularyTestCase),
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""Functions to aid the calling of bibutils command line programs.

See <http://www.scripps.edu/~cdputnam/software/bibutils/>.
"""

__docformat__ = 'reStructuredText'
__author__  = 'Tom Gross <itconsense@gmail.com>'

# Standard library imports
import logging

# Local imports
from bibliograph.core.utils import bin_search


log = logging.getLogger('bibliograph.core')

commands = {'bib2xml':'bib2xml -un',
            'copac2xml':'copac2xml',
            'end2xml':'end2xml',
            'isi2xml':'isi2xml',
            'med2xml':'med2xml',
            'ris2xml':'ris2xml',
            'xml2bib':'xml2bib',
            'xml2end':'xml2end',
            'xml2ris':'xml2ris',
            'bib2bib':'bib2xml | xml2bib',
            'copac2bib':'copac2xml | xml2bib ',
            'end2bib':'end2xml -i utf8 | xml2bib -nl -nb -o utf8 ',
            'isi2bib':'isi2xml | xml2bib -nl -nb -o utf8 ',
            'med2bib':'med2xml | xml2bib -nl -nb -o utf8 ',
            'ris2bib':'ris2xml -i utf8 | xml2bib -nl -nb -o utf8',
            'bib2end':'bib2xml | xml2end -nb -o unicode',
            'bib2ris':'bib2xml | xml2ris -nb -o unicode',
            'copac2end':'copac2xml | xml2end ',
            'isi2end':'isi2xml | xml2end ',
            'med2end':'med2xml | xml2end ',
            'ris2end':'ris2xml | xml2end ',
            'end2ris':'end2xml | xml2ris ',
            'copac2ris':'copac2xml | xml2ris ',
            'isi2ris':'isi2xml | xml2ris ',
            'med2ris':'med2xml | xml2ris ',
            }

def _getKey(source_format, target_format):
    return '2'.join([source_format, target_format])


def _hasCommands(command):
    """ Check if a collection of piped commands is available

        >>> _hasCommands('python -o|python -wrt')
        True

        >>> _hasCommands(' something_strange -m | python')
        False

    """
    for cmd in command.split('|'):
        cmd = cmd.strip()
        if ' ' in cmd:
            cmd = cmd[:cmd.find(' ')]
        if bin_search(cmd, False) is False:
            log.warn('Command %s not found in search path!', cmd)
            return False
    return True

_marker = object()
def _getCommand(source_format, target_format, default=_marker):
    key = _getKey(source_format, target_format)
    command = commands.get(key, None)
    if command is None:
        raise ValueError, "No transformation from '%s' to '%s' found." \
              % (source_format, target_format)

    if not _hasCommands(command):
        if default is _marker:
            raise LookupError, "Command %s not found." % command
        else:
            return default
    return command



# -*- coding: utf-8 -*-
###############################################################################
# $Copy$
###############################################################################
""" Collection of python utility-methodes commonly used by other
    bibliograph packages.

$Id: utils.py 114461 2010-04-01 16:16:54Z ajung $
"""
__docformat__ = 'reStructuredText'
__author__  = 'Tom Gross <itconsense@gmail.com>'

# Python imports
import os
import re
import string
import sys

# My imports ;-)
from bibliograph.core.encodings import _utf8enc2latex_mapping

###############################################################################

_default_encoding = 'utf-8'
_entity_mapping = {'&mdash;':'{---}',
                   '&ndash;':'{--}',
                   }

###############################################################################
# a poor-mans approach to fixing unicode issues :-(

def _encode(s, encoding=_default_encoding):
    ur""" Try to encode a string

        >>> from bibliograph.core.utils import _encode

        ASCII is ASCII is ASCII ...
        >>> _encode(u'ascii', 'utf-8')
        'ascii'

        This is normal
        >>> _encode(u'öl', 'utf-8')
        '\xc3\xb6l'

        Don't fail on this ...
        >>> _encode('öl', 'utf-8')
        '\xc3\xb6l'

        Don't fail on this also
        >>> _encode(None, 'utf-8')
        ''

        Still throw an exception on unknown encodings
        >>> _encode(u'öl', 'bogus')
        Traceback (most recent call last):
        ...
        LookupError: unknown encoding: bogus

    """
    if not s:
        return ''
    try:
        return s.encode(encoding)
    except (TypeError, UnicodeDecodeError, ValueError):
        return s

def _decode(s, encoding=_default_encoding):
    ur""" Try to decode a string

        >>> from bibliograph.core.utils import _decode

        ASCII is ASCII is ASCII ...
        >>> _decode('ascii', 'utf-8')
        u'ascii'

        This is normal
        >>> _decode('öl', 'utf-8')
        u'\xf6l'

        Don't fail on this ...
        >>> _decode(u'öl', 'utf-8')
        u'\xf6l'

        Still throw an exception on unknown encodings
        >>> _decode('öl', 'bogus')
        Traceback (most recent call last):
        ...
        LookupError: unknown encoding: bogus

    """
    try:
        return unicode(s, encoding)
    except (TypeError, UnicodeDecodeError, ValueError):
        return s

###############################################################################

VALIDIDPAT = re.compile('[ "@\',\\#}{~%&$^]')

def _validKey(entry):
    """
    uses the Zope object id but
    removes characters not allowed in BibTeX keys

    >>> from bibliograph.core.utils import _validKey
    >>> _validKey(DummyEntry('Foo Bar'))
    'FooBar'

    >>> _validKey(DummyEntry('my@id'))
    'myid'

    """
    # be forward compatible to zope3 contained objects
    raw_id = getattr(entry, '__name__', '')
    if not raw_id:
        raw_id = entry.getId()

    # This substitution is based on the description of cite key restrictions at
    # http://bibdesk.sourceforge.net/manual/BibDesk%20Help_2.html
    return VALIDIDPAT.sub('', raw_id)

###############################################################################

def AuthorURLs(entry):
    """a string with all the known author's URLs;
    helper method for bibtex output"""
    a_URLs = ''
    for a in entry.getAuthors():
        url = a.get('homepage', ' ')
        a_URLs += "%s and " % url
    return a_URLs[:-5]

###############################################################################

def _braceUppercase(text):
    """ Convert uppercase letters to bibtex encoded uppercase

        >>> from bibliograph.core.utils import _braceUppercase
        >>> _braceUppercase('foo bar')
        'foo bar'

        >>> _braceUppercase('Foo Bar')
        '{F}oo {B}ar'
    """
    for uc in string.uppercase:
        text = text.replace(uc, r'{%s}' % uc)
    return text

###############################################################################

def _normalize(text, resolve_unicode=True):
    text.replace('\\', '\\\\')
    text = _resolveEntities(text)
    if resolve_unicode:
        text = _resolveUnicode(text)
    return _escapeSpecialCharacters(text)

###############################################################################

def _resolveEntities(text):
    for entity in _entity_mapping.keys():
        text = text.replace(entity, _entity_mapping[entity])
    return text

###############################################################################

def _resolveUnicode(text):
    for unichar in _utf8enc2latex_mapping.keys():
        text = _encode(_decode(text).replace(unichar,
                                             _utf8enc2latex_mapping[unichar]))
    return _encode(_decode(text).replace(r'$}{$',''))

###############################################################################

def _escapeSpecialCharacters(text):
    """
    latex escaping some (not all) special characters
    """
    text.replace('\\', '\\\\')
    escape = ['~', '#', '&', '%', '_']
    for c in escape:
        text = text.replace(c, '\\' + c )
    return text


###############################################################################

def title_or_id(context):
    """ Return the title or id.

        Works with Zope2 and Zope3
    """
    title = getattr(context, 'title', '')
    if not title:
        if hasattr(context, '__name__'):
            title = getattr(context, '__name__', '')
        elif hasattr(context, 'getId'):
            title = context.getId()
    return title

###############################################################################

def _convertToOutputEncoding(export_text,
                             input_encoding=None,
                             output_encoding='utf-8'):
    """ Convert the renderer's result to the output encoding
    """
    if input_encoding:
        return _encode(_decode(export_text, input_encoding), output_encoding)
    return _encode(_decode(export_text), output_encoding)

###############################################################################

class MissingBinary(Exception): pass
_marker = object()

def bin_search(binary, default=_marker):
    """ Search the bin_search_path for a given binary

        Returns its fullname or raises MissingBinary exception

        We assume we have a python(.exe) installed anywhere in
        the path. This seems one of the safest test.
        >>> from bibliograph.core.utils import bin_search
        >>> bin_search('python') != ''
        True

        >>> bin_search('a_completely_stupid_command')
        Traceback (most recent call last):
        ...
        MissingBinary: Unable to find binary "a_completely_stupid_command" ...

        >>> bin_search('a_completely_stupid_command', '/bin/default')
        '/bin/default'

        Let's see if the additional path is searched.
        First we create a temporary directory and add it to the
        environment.
        >>> import os, stat, tempfile
        >>> dir = tempfile.mkdtemp()
        >>> os.environ['BIBUTILS_PATH'] = dir

        Now create a dummy executable we want to find.
        >>> exe = os.path.join(dir, '_stub_testing_file')
        >>> f = open(exe, 'w')
        >>> f.write('foobar')
        >>> f.close()
        >>> os.chmod(exe, stat.S_IXUSR | stat.S_IRUSR)

        Do the search and compare.
        >>> bin_search('_stub_testing_file') == exe
        True

        Cleanup.
        >>> os.unlink(exe)
        >>> os.rmdir(dir)
    """
    mode   = os.R_OK | os.X_OK
    envPath = os.environ['PATH']
    customPath = os.environ.get('BIBUTILS_PATH', '')
    searchPath = os.pathsep.join([customPath, envPath])
    bin_search_path = [path for path in searchPath.split(os.pathsep)
                       if os.path.isdir(path)]

    if sys.platform == 'win32':
        extensions = ('.exe', '.com', '.bat', )
    else:
        extensions = ()

    for path in bin_search_path:
        for ext in ('', ) + extensions:
            pathbin = os.path.join(path, binary) + ext
            if os.access(pathbin, mode) == 1:
                return pathbin

    if default is _marker:
        raise MissingBinary('Unable to find binary "%s" in %s w' %
                           (binary, os.pathsep.join(bin_search_path)))
    else:
        return default

